#include "timer.h"
#include <stdio.h>

static uint16_t tickval=0;

typedef struct {
  TIMER_STATE_T timerStatus;
  uint8_t presetval;
}TIMER_T;

#define MAX_TIMERS 5

TIMER_T timers[MAX_TIMERS]={{UNUSED,0}};

//
// Functions for all timers
//

uint8_t timerCreate(void){
  uint8_t n=0;
  while(n<MAX_TIMERS){
    if(timers[n].timerStatus==UNUSED){
        timers[n].timerStatus = OFF;
        return n;
    }
  }
  return 255; // no timers left
}

void timerErase(uint8_t t_id){
  // not jet impl.
  
}

//
// Timer functions for a single timer
//

// tick is called every 100ms
uint8_t tick(uint8_t t_id){
	
	uint8_t retVal=0;
  uint8_t n=0;

  if(t_id>(MAX_TIMERS-1)) return 0;
    
  if(timers[n].timerStatus==ON){		
		
		if(tickval<timers[n].presetval){
			tickval++;
			printf("Timer %dms left\n",(timers[n].presetval - (tickval))*100);
		}
			
		if(tickval==timers[n].presetval){
			printf("Timer::timeout\n");
			timers[n].timerStatus=OFF;
			retVal=1;
		}
  }
	return retVal;
}

void timerSet(uint8_t t_id, unsigned char hms){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].presetval=hms;
}

void timerPause(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].timerStatus=PAUSE;
}

void timerCont(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].timerStatus=ON;
}

void timerClear(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].presetval=0;
}

uint8_t timerPreset(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return 0;
  return timers[t_id].presetval;
}

TIMER_STATE_T timerStart(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return UNKNOWN;

	printf("Timer::start\n");
	
	if(timers[t_id].presetval!=0){
		tickval=0;
	  timers[t_id].timerStatus = ON;
		return ON;
	}else{
		timers[t_id].timerStatus = OFF;
		return OFF;
	}
}

void timerStop(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return;

	printf("Timer::stop\n");
  timers[t_id].timerStatus=OFF;
}

TIMER_STATE_T timerStatus(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return UNKNOWN;

	return timers[t_id].timerStatus;
}

void timerInc(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;

	if(timers[t_id].presetval<60){
		timers[t_id].presetval++;
		printf("Timer::inc -> preset=%d\n",timers[t_id].presetval);
	}
}

void timerDec(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return;
  
  if(timers[t_id].presetval>0)
		timers[t_id].presetval--;
		printf("Timer::dec -> preset=%d\n",timers[t_id].presetval);
}
